/*
 * Java
 *
 * Copyright 2019-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.library.iot.rcommand.bluetooth.commands;

import java.io.IOException;

import ej.library.iot.rcommand.bluetooth.Commands;
import ej.rcommand.CommandReader;
import ej.rcommand.CommandSender;
import ej.rcommand.synchronous.Command;

public class StartAdvertisingCommand extends Command<Boolean> {

	private final byte[] advertisementData;
	private final int advertisementDataSize;

	public StartAdvertisingCommand(byte[] advertisementData, int advertisementDataSize) {
		this.advertisementData = advertisementData;
		this.advertisementDataSize = advertisementDataSize;
	}

	@Override
	public String getName() {
		return Commands.BLUETOOTH_START_ADVERTISING;
	}

	@Override
	public void writeBody(CommandSender sender) throws IOException {
		sender.sendByteArray(this.advertisementData, 0, this.advertisementDataSize);
	}

	@Override
	public Boolean readResponseInternal(CommandReader reader) throws IOException {
		return Boolean.valueOf(reader.readInt() == Commands.OK);
	}
}
